@@ -25,6 +25,7 @@ OVN_ROUTER_NAME_EXT_ID_KEY = 'neutron:router_name'
 OVN_ROUTER_IS_EXT_GW = 'neutron:is_ext_gw'
 OVN_GW_PORT_EXT_ID_KEY = 'neutron:gw_port_id'
 OVN_SUBNET_EXT_ID_KEY = 'neutron:subnet_id'
+OVN_SUBNET_EXT_IDS_KEY = 'neutron:subnet_ids'
 OVN_PHYSNET_EXT_ID_KEY = 'neutron:provnet-physical-network'
 OVN_NETTYPE_EXT_ID_KEY = 'neutron:provnet-network-type'
 OVN_SEGID_EXT_ID_KEY = 'neutron:provnet-segmentation-id'
@@ -33,6 +34,7 @@ OVN_DEVID_EXT_ID_KEY = 'neutron:device_id'
 OVN_CIDRS_EXT_ID_KEY = 'neutron:cidrs'
 OVN_FIP_EXT_ID_KEY = 'neutron:fip_id'
 OVN_FIP_PORT_EXT_ID_KEY = 'neutron:fip_port_id'
+OVN_FIP_EXT_MAC_KEY = 'neutron:fip_external_mac'
 OVN_REV_NUM_EXT_ID_KEY = 'neutron:revision_number'
 OVN_QOS_POLICY_EXT_ID_KEY = 'neutron:qos_policy_id'
 OVN_SG_IDS_EXT_ID_KEY = 'neutron:security_group_ids'
@@ -123,3 +125,7 @@ MAINTENANCE_CREATE_UPDATE_TYPE_ORDER = {
 # task: Leaf ones first and roots at the end.
 MAINTENANCE_DELETE_TYPE_ORDER = {
     t: n for n, t in enumerate(reversed(_TYPES_PRIORITY_ORDER), 1)}
+
+# The addresses field to set in the logical switch port which has a
+# peer router port (connecting to the logical router).
+DEFAULT_ADDR_FOR_LSP_WITH_PEER = 'router'
