@@ -30,17 +30,18 @@ ML2_SUPPORTED_API_EXTENSIONS_NEUTRON_L3 = [
 ML2_SUPPORTED_API_EXTENSIONS_OVN_L3 = [
     'router',
     'extraroute',
-    'ext-gw-mode'
+    'ext-gw-mode',
+    'pagination',
+    'sorting',
+    'project-id',
 ]
 ML2_SUPPORTED_API_EXTENSIONS = [
     'address-scope',
-    'agent',
     'allowed-address-pairs',
     'auto-allocated-topology',
     'availability_zone',
     'binding',
     'default-subnetpools',
-    'dhcp_agent_scheduler',
     'external-net',
     'extra_dhcp_opt',
     'multi-provider',
@@ -51,10 +52,11 @@ ML2_SUPPORTED_API_EXTENSIONS = [
     'provider',
     'quotas',
     'rbac-policies',
-    'revisions',
+    'standard-attr-revisions',
     'security-group',
     'standard-attr-description',
     'subnet_allocation',
-    'tag',
-    'timestamp_core',
+    'standard-attr-tag',
+    'standard-attr-timestamp',
+    'trunk',
 ]
