@@ -24,6 +24,7 @@ from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net as pnet
 from neutron_lib import constants as const
 from neutron_lib import context as n_context
+from neutron_lib import exceptions as n_exc
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
 from neutron_lib.utils import helpers
@@ -578,16 +579,19 @@ class OVNClient(object):
                    'logical_ip': floatingip['fixed_ip_address'],
                    'external_ip': floatingip['floating_ip_address']}
 
+        if config.is_ovn_distributed_floating_ip():
+            port = self._plugin.get_port(
+                context, fip_db['floating_port_id'])
+            columns['logical_port'] = floatingip['port_id']
+            ext_ids[ovn_const.OVN_FIP_EXT_MAC_KEY] = port['mac_address']
+            if self._nb_idl.lsp_get_up(floatingip['port_id']).execute():
+                columns['external_mac'] = port['mac_address']
+
         # TODO(dalvarez): remove this check once the minimum OVS required
         # version contains the column (when OVS 2.8.2 is released).
         if self._nb_idl.is_col_present('NAT', 'external_ids'):
             columns['external_ids'] = ext_ids
 
-        if config.is_ovn_distributed_floating_ip():
-            port = self._plugin.get_port(
-                context, fip_db['floating_port_id'])
-            columns['external_mac'] = port['mac_address']
-            columns['logical_port'] = floatingip['port_id']
         commands.append(func(*nat_rule_args, **columns))
         self._transaction(commands, txn=txn)
 
@@ -766,7 +770,7 @@ class OVNClient(object):
         gw_info = self._get_gw_info(context, router)
         gw_port_id = router['gw_port_id']
         port = self._plugin.get_port(context, gw_port_id)
-        self.create_router_port(router_id, port, txn=txn)
+        self._create_lrouter_port(router_id, port, txn=txn)
 
         # TODO(lucasagomes): Remove this check after OVS 2.8.2 is tagged
         # (prior to that, the external_ids column didn't exist in this
@@ -1022,11 +1026,19 @@ class OVNClient(object):
             return extnet.get(pnet.PHYSICAL_NETWORK)
 
     def _gen_router_port_ext_ids(self, port):
-        return {
+        ext_ids = {
             ovn_const.OVN_REV_NUM_EXT_ID_KEY: str(utils.get_revision_number(
-                port, ovn_const.TYPE_ROUTER_PORTS))}
+                port, ovn_const.TYPE_ROUTER_PORTS)),
+            ovn_const.OVN_SUBNET_EXT_IDS_KEY:
+                ' '.join(utils.get_port_subnet_ids(port))}
 
-    def create_router_port(self, router_id, port, txn=None):
+        router_id = port.get('device_id')
+        if router_id:
+            ext_ids[ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY] = router_id
+
+        return ext_ids
+
+    def _create_lrouter_port(self, router_id, port, txn=None):
         """Create a logical router port."""
         lrouter = utils.ovn_name(router_id)
         networks, ipv6_ra_configs = (
@@ -1044,8 +1056,22 @@ class OVNClient(object):
                 candidates=candidates)
             if selected_chassis:
                 columns['gateway_chassis'] = selected_chassis
+
+        lsp_address = ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER
         if ipv6_ra_configs:
             columns['ipv6_ra_configs'] = ipv6_ra_configs
+            # TODO(numans)
+            # If we set router port's peer logical switch port with
+            # addresses=ovn_const.DEFAULT_ADDRESS_FOR_LSP_WITH_PEER_ROUTER_PORT
+            # ovn-northd is adding IPv6 Neighbor Adv flow in the logical switch
+            # pipeline using the 'nd_na' action.
+            # 'nd_na' action doesn't set the Router bit in the 'flags' field
+            # of the Neigh Adv packet in response to the Neigh Solicitation
+            # packet for the router IPv6 address. See bug #1788684.
+            # Until it is fixed in core OVN, set lsp_address to the mac_address
+            # of port so that we don't set addresses=router. Once that is fixed
+            # remove the below line.
+            lsp_address = [port['mac_address']]
 
         commands = [
             self._nb_idl.add_lrouter_port(
@@ -1057,51 +1083,166 @@ class OVNClient(object):
                 external_ids=self._gen_router_port_ext_ids(port),
                 **columns),
             self._nb_idl.set_lrouter_port_in_lswitch_port(
-                port['id'], lrouter_port_name, is_gw_port=is_gw_port)]
+                port['id'], lrouter_port_name, is_gw_port=is_gw_port,
+                lsp_address=lsp_address)]
         self._transaction(commands, txn=txn)
-        # NOTE(mangelajo): we don't bump the revision here, but we do
-        # in the higher level add_router_interface function, because
-        # we can't consider it done until the Nat rules are updated
 
-    def update_router_port(self, port, bump_db_rev=True, if_exists=False):
+    def create_router_port(self, router_id, router_interface):
+        context = n_context.get_admin_context()
+        port = self._plugin.get_port(context, router_interface['port_id'])
+        with self._nb_idl.transaction(check_error=True) as txn:
+            multi_prefix = False
+            if (len(router_interface.get('subnet_ids', [])) == 1 and
+                    len(port['fixed_ips']) > 1):
+
+                # NOTE(lizk) It's adding a subnet onto an already
+                # existing router interface port, try to update lrouter port
+                # 'networks' column.
+                self._update_lrouter_port(port, txn=txn)
+                multi_prefix = True
+            else:
+                self._create_lrouter_port(router_id, port, txn=txn)
+
+            router = self._l3_plugin.get_router(context, router_id)
+            if router.get(l3.EXTERNAL_GW_INFO):
+                cidr = None
+                for fixed_ip in port['fixed_ips']:
+                    subnet = self._plugin.get_subnet(context,
+                                                     fixed_ip['subnet_id'])
+                    if multi_prefix:
+                        if 'subnet_id' in router_interface:
+                            if subnet['id'] != router_interface['subnet_id']:
+                                continue
+                    if subnet['ip_version'] == 4:
+                        cidr = subnet['cidr']
+
+                if utils.is_snat_enabled(router) and cidr:
+                    self.update_nat_rules(router, networks=[cidr],
+                                          enable_snat=True, txn=txn)
+
+        db_rev.bump_revision(port, ovn_const.TYPE_ROUTER_PORTS)
+
+    def _update_lrouter_port(self, port, if_exists=False, txn=None):
         """Update a logical router port."""
         networks, ipv6_ra_configs = (
             self._get_nets_and_ipv6_ra_confs_for_router_port(
                 port['fixed_ips']))
 
-        lrouter_port_name = utils.ovn_lrouter_port_name(port['id'])
+        lsp_address = ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER
+        if ipv6_ra_configs:
+            # TODO(numans)
+            # If we set router port's peer logical switch port with
+            # addresses=ovn_const.DEFAULT_ADDRESS_FOR_LSP_WITH_PEER_ROUTER_PORT
+            # ovn-northd is adding IPv6 Neighbor Adv flow in the logical switch
+            # pipeline using the 'nd_na' action.
+            # 'nd_na' action doesn't set the Router bit in the 'flags' field
+            # of the Neigh Adv packet in response to the Neigh Solicitation
+            # packet for the router IPv6 address. See bug #1788684.
+            # Until it is fixed in core OVN, set lsp_address to the mac_address
+            # of port so that we don't set addresses=router. Once that is fixed
+            # remove this if condition.
+            lsp_address = [port['mac_address']]
+
+        lrp_name = utils.ovn_lrouter_port_name(port['id'])
         update = {'networks': networks, 'ipv6_ra_configs': ipv6_ra_configs}
         is_gw_port = const.DEVICE_OWNER_ROUTER_GW == port.get(
             'device_owner')
+        commands = [
+            self._nb_idl.update_lrouter_port(
+                name=lrp_name,
+                external_ids=self._gen_router_port_ext_ids(port),
+                if_exists=if_exists,
+                **update),
+            self._nb_idl.set_lrouter_port_in_lswitch_port(
+                port['id'], lrp_name, is_gw_port=is_gw_port,
+                lsp_address=lsp_address)]
+
+        self._transaction(commands, txn=txn)
+
+    def update_router_port(self, port, if_exists=False):
+        lrp_name = utils.ovn_lrouter_port_name(port['id'])
         check_rev_cmd = self._nb_idl.check_revision_number(
-            lrouter_port_name, port, ovn_const.TYPE_ROUTER_PORTS)
+            lrp_name, port, ovn_const.TYPE_ROUTER_PORTS)
         with self._nb_idl.transaction(check_error=True) as txn:
             txn.add(check_rev_cmd)
-            txn.add(self._nb_idl.update_lrouter_port(
-                    name=lrouter_port_name,
-                    external_ids=self._gen_router_port_ext_ids(port),
-                    if_exists=if_exists,
-                    **update))
-            txn.add(self._nb_idl.set_lrouter_port_in_lswitch_port(
-                    port['id'], lrouter_port_name, is_gw_port=is_gw_port))
+            self._update_lrouter_port(port, if_exists=if_exists, txn=txn)
 
-        if bump_db_rev and check_rev_cmd.result == ovn_const.TXN_COMMITTED:
+        if check_rev_cmd.result == ovn_const.TXN_COMMITTED:
             db_rev.bump_revision(port, ovn_const.TYPE_ROUTER_PORTS)
-        else:
-            # NOTE(mangelajo): we don't bump the revision here, but we do
-            # in the higher level add_router_interface function, because
-            # we can't consider it done until the Nat rules are updated
-            pass
 
-    def delete_router_port(self, port_id, router_id=None):
+    def _delete_lrouter_port(self, port_id, router_id=None, txn=None):
         """Delete a logical router port."""
-        with self._nb_idl.transaction(check_error=True) as txn:
-            txn.add(self._nb_idl.lrp_del(
-                utils.ovn_lrouter_port_name(port_id),
-                utils.ovn_name(router_id) if router_id else None,
-                if_exists=True))
+        commands = [self._nb_idl.lrp_del(
+            utils.ovn_lrouter_port_name(port_id),
+            utils.ovn_name(router_id) if router_id else None,
+            if_exists=True)]
+        self._transaction(commands, txn=txn)
         db_rev.delete_revision(port_id, ovn_const.TYPE_ROUTER_PORTS)
 
+    def delete_router_port(self, port_id, router_id=None, subnet_ids=None):
+        try:
+            ovn_port = self._nb_idl.lookup(
+                'Logical_Router_Port', utils.ovn_lrouter_port_name(port_id))
+        except idlutils.RowNotFound:
+            return
+
+        subnet_ids = subnet_ids or []
+        context = n_context.get_admin_context()
+        port_removed = False
+        with self._nb_idl.transaction(check_error=True) as txn:
+            port = None
+            try:
+                port = self._plugin.get_port(context, port_id)
+                # The router interface port still exists, call ovn to
+                # update it
+                self._update_lrouter_port(port, txn=txn)
+            except n_exc.PortNotFound:
+                # The router interface port doesn't exist any more,
+                # we will call ovn to delete it once we remove the snat
+                # rules in the router itself if we have to
+                port_removed = True
+
+            router_id = router_id or ovn_port.external_ids.get(
+                ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY)
+            if not router_id:
+                router_id = port.get('device_id')
+
+            router = None
+            if router_id:
+                router = self._l3_plugin.get_router(context, router_id)
+
+            if not router.get(l3.EXTERNAL_GW_INFO):
+                if port_removed:
+                    self._delete_lrouter_port(port_id, router_id, txn=txn)
+                return
+
+            if not subnet_ids:
+                subnet_ids = ovn_port.external_ids.get(
+                    ovn_const.OVN_SUBNET_EXT_IDS_KEY, [])
+                subnet_ids = subnet_ids.split()
+            elif port:
+                subnet_ids = utils.get_port_subnet_ids(port)
+
+            cidr = None
+            for sid in subnet_ids:
+                subnet = self._plugin.get_subnet(context, sid)
+                if subnet['ip_version'] == 4:
+                    cidr = subnet['cidr']
+                    break
+
+            if router and utils.is_snat_enabled(router) and cidr:
+                self.update_nat_rules(
+                    router, networks=[cidr], enable_snat=False, txn=txn)
+
+            # NOTE(mangelajo): If the port doesn't exist anymore, we
+            # delete the router port as the last operation and update the
+            # revision database to ensure consistency
+            if port_removed:
+                self._delete_lrouter_port(port_id, router_id, txn=txn)
+            else:
+                # otherwise, we just update the revision database
+                db_rev.bump_revision(port, ovn_const.TYPE_ROUTER_PORTS)
+
     def update_nat_rules(self, router, networks, enable_snat, txn=None):
         """Update the NAT rules in a logical router."""
         context = n_context.get_admin_context()
@@ -1261,9 +1402,15 @@ class OVNClient(object):
             options['server_mac'] = n_net.get_random_mac(
                 cfg.CONF.base_mac.split(':'))
 
-        if subnet['dns_nameservers']:
-            dns_servers = '{%s}' % ', '.join(subnet['dns_nameservers'])
-            options['dns_server'] = dns_servers
+        dns_servers = (subnet.get('dns_nameservers') or
+                       config.get_dns_servers() or
+                       utils.get_system_dns_resolvers())
+        if dns_servers:
+            options['dns_server'] = '{%s}' % ', '.join(dns_servers)
+        else:
+            LOG.warning("No relevant dns_servers defined for subnet %s. Check "
+                        "the /etc/resolv.conf file",
+                        subnet['id'])
 
         routes = []
         if metadata_port_ip:
@@ -1374,11 +1521,28 @@ class OVNClient(object):
         txn.add(self._nb_idl.add_dhcp_options(subnet['id'], **new_options))
         dhcp_options = self._nb_idl.get_subnet_dhcp_options(
             subnet['id'], with_ports=True)
+
+        # When a subnet dns_nameserver is updated, then we should update
+        # the port dhcp options for ports (with no port specific dns_server
+        # defined).
+        if 'options' in new_options and 'options' in original_options:
+            orig_dns_server = original_options['options'].get('dns_server')
+            new_dns_server = new_options['options'].get('dns_server')
+            dns_server_changed = (orig_dns_server != new_dns_server)
+        else:
+            dns_server_changed = False
+
         for opt in dhcp_options['ports']:
             if not new_options.get('options'):
                 continue
             options = dict(new_options['options'])
+            p_dns_server = opt['options'].get('dns_server')
+            if dns_server_changed and (orig_dns_server == p_dns_server):
+                # If port has its own dns_server option defined, then
+                # orig_dns_server and p_dns_server will not match.
+                opt['options']['dns_server'] = new_dns_server
             options.update(opt['options'])
+
             port_id = opt['external_ids']['port_id']
             txn.add(self._nb_idl.add_dhcp_options(
                 subnet['id'], port_id=port_id, options=options))
@@ -1417,7 +1581,7 @@ class OVNClient(object):
     def delete_subnet(self, subnet_id):
         with self._nb_idl.transaction(check_error=True) as txn:
             self._remove_subnet_dhcp_options(subnet_id, txn)
-        db_rev.delete_revision(subnet_id, ovn_const.TYPE_FLOATINGIPS)
+        db_rev.delete_revision(subnet_id, ovn_const.TYPE_SUBNETS)
 
     def create_security_group(self, security_group):
         with self._nb_idl.transaction(check_error=True) as txn:
