@@ -354,13 +354,18 @@ class UpdateLRouterPortCommand(command.BaseCommand):
             msg = _("Logical Router Port %s does not exist") % self.name
             raise RuntimeError(msg)
 
-        if lrouter_port:
-            for col, val in self.columns.items():
-                if col == 'gateway_chassis':
-                    col, val = _add_gateway_chassis(self.api, txn, self.name,
-                                                    val)
-                setattr(lrouter_port, col, val)
-            return
+        # TODO(lucasagomes): Remove this check once we drop the support
+        # for OVS versions <= 2.8
+        ipv6_ra_configs_supported = self.api.is_col_present(
+            'Logical_Router_Port', 'ipv6_ra_configs')
+        for col, val in self.columns.items():
+            if col == 'ipv6_ra_configs' and not ipv6_ra_configs_supported:
+                continue
+
+            if col == 'gateway_chassis':
+                col, val = _add_gateway_chassis(self.api, txn, self.name,
+                                                val)
+            setattr(lrouter_port, col, val)
 
 
 class DelLRouterPortCommand(command.BaseCommand):
@@ -391,12 +396,14 @@ class DelLRouterPortCommand(command.BaseCommand):
 
 
 class SetLRouterPortInLSwitchPortCommand(command.BaseCommand):
-    def __init__(self, api, lswitch_port, lrouter_port, is_gw_port, if_exists):
+    def __init__(self, api, lswitch_port, lrouter_port, is_gw_port,
+                 if_exists, lsp_address):
         super(SetLRouterPortInLSwitchPortCommand, self).__init__(api)
         self.lswitch_port = lswitch_port
         self.lrouter_port = lrouter_port
         self.is_gw_port = is_gw_port
         self.if_exists = if_exists
+        self.lsp_address = lsp_address
 
     def run_idl(self, txn):
         try:
@@ -414,7 +421,7 @@ class SetLRouterPortInLSwitchPortCommand(command.BaseCommand):
             options[ovn_const.OVN_GATEWAY_NAT_ADDRESSES_KEY] = 'router'
         setattr(port, 'options', options)
         setattr(port, 'type', 'router')
-        setattr(port, 'addresses', 'router')
+        setattr(port, 'addresses', self.lsp_address)
 
 
 class AddACLCommand(command.BaseCommand):
