@@ -303,11 +303,13 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         return cmd.DelLRouterPortCommand(self, name, lrouter,
                                          if_exists)
 
-    def set_lrouter_port_in_lswitch_port(self, lswitch_port, lrouter_port,
-                                         is_gw_port=False, if_exists=True):
+    def set_lrouter_port_in_lswitch_port(
+            self, lswitch_port, lrouter_port, is_gw_port=False, if_exists=True,
+            lsp_address=ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER):
         return cmd.SetLRouterPortInLSwitchPortCommand(self, lswitch_port,
                                                       lrouter_port, is_gw_port,
-                                                      if_exists)
+                                                      if_exists,
+                                                      lsp_address)
 
     def add_acl(self, lswitch, lport, **columns):
         return cmd.AddACLCommand(self, lswitch, lport, **columns)
@@ -640,6 +642,8 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
 
     def get_lrouter_port(self, lrp_name):
         # TODO(mangelajo): Implement lrp_get() ovsdbapp and use from here
+        if uuidutils.is_uuid_like(lrp_name):
+            lrp_name = utils.ovn_lrouter_port_name(lrp_name)
         lrp = self.db_find_rows('Logical_Router_Port', ('name', '=', lrp_name))
         result = lrp.execute(check_error=True)
         return result[0] if result else None
