@@ -114,6 +114,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                         '{169.254.169.254/32,10.0.0.2, 0.0.0.0/0,10.0.0.1}',
                         'server_id': '10.0.0.1',
                         'server_mac': '01:02:03:04:05:06',
+                        'dns_server': '{10.10.10.10}',
                         'lease_time': str(12 * 60 * 60),
                         'mtu': str(n1['network']['mtu']),
                         'router': n1_s1['subnet']['gateway_ip']}})
@@ -153,7 +154,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                                      for f in port['port']['fixed_ips']])
                 hname = 'n1-' + p
                 self.expected_dns_records[0]['records'][hname] = port_ips
-                hname = 'n1-' + p + '.ovn.test.'
+                hname = 'n1-' + p + '.ovn.test'
                 self.expected_dns_records[0]['records'][hname] = port_ips
             if p == 'p1':
                 fake_subnet = {'cidr': '11.11.11.11/24'}
@@ -243,6 +244,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                         '{169.254.169.254/32,10.0.0.2, 0.0.0.0/0,10.0.0.1}',
                         'server_id': '10.0.0.1',
                         'server_mac': '01:02:03:04:05:06',
+                        'dns_server': '{10.10.10.10}',
                         'lease_time': str(12 * 60 * 60),
                         'mtu': str(n1['network']['mtu']),
                         'router': n1_s1['subnet']['gateway_ip'],
@@ -276,6 +278,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                         '{169.254.169.254/32,20.0.0.2, 0.0.0.0/0,20.0.0.1}',
                         'server_id': '20.0.0.1',
                         'server_mac': '01:02:03:04:05:06',
+                        'dns_server': '{10.10.10.10}',
                         'lease_time': str(12 * 60 * 60),
                         'mtu': str(n2['network']['mtu']),
                         'router': n2_s1['subnet']['gateway_ip']}})
@@ -484,7 +487,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                                      for f in port['port']['fixed_ips']])
                 hname = 'n4-' + p
                 self.expected_dns_records[1]['records'][hname] = port_ips
-                hname = 'n4-' + p + '.ovn.test.'
+                hname = 'n4-' + p + '.ovn.test'
                 self.expected_dns_records[1]['records'][hname] = port_ips
 
             n4_port_dict[p] = port['port']['id']
@@ -583,6 +586,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
             'options': {'classless_static_route':
                         '{169.254.169.254/32,30.0.0.2, 0.0.0.0/0,30.0.0.1}',
                         'server_id': '30.0.0.1',
+                        'dns_server': '{10.10.10.10}',
                         'server_mac': dhcp_mac_v4,
                         'lease_time': str(12 * 60 * 60),
                         'mtu': str(n3['network']['mtu']),
@@ -1087,7 +1091,6 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                 mac_address = ''
                 fip_port = ''
                 if fip['id'] in fip_macs:
-                    mac_address = fip_macs[fip['id']]
                     fip_port = fip['port_id']
                 db_nats[fip['router_id']].append(
                     fip['floating_ip_address'] + fip['fixed_ip_address'] +
