@@ -20,6 +20,7 @@ from neutron.tests.unit.api import test_extensions
 from neutron.tests.unit.extensions import test_extraroute
 from neutron.tests.unit.extensions import test_l3
 from neutron.tests.unit.extensions import test_l3_ext_gw_mode as test_l3_gw
+from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import resources
 from neutron_lib import constants
@@ -64,7 +65,9 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             'name': 'lrp-router-port-id',
             'may_exist': True,
             'networks': ['10.0.0.100/24'],
-            'external_ids': {ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1'}}
+            'external_ids': {
+                ovn_const.OVN_SUBNET_EXT_IDS_KEY: 'subnet-id',
+                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1'}}
         self.fake_router_ports = [self.fake_router_port]
         self.fake_subnet = {'id': 'subnet-id',
                             'ip_version': 4,
@@ -116,7 +119,9 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             'name': 'lrp-gw-port-id',
             'networks': ['192.168.1.1/24'],
             'may_exist': True,
-            'external_ids': {ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1'},
+            'external_ids': {
+                ovn_const.OVN_SUBNET_EXT_IDS_KEY: 'ext-subnet-id',
+                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1'},
             'gateway_chassis': ['hv1']}
         self.fake_floating_ip_attrs = {'floating_ip_address': '192.168.0.10',
                                        'fixed_ip_address': '10.0.0.10'}
@@ -212,8 +217,9 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         self.l3_inst._ovn.add_lrouter_port.assert_called_once_with(
             **self.fake_router_port_assert)
         self.l3_inst._ovn.set_lrouter_port_in_lswitch_port.\
-            assert_called_once_with('router-port-id', 'lrp-router-port-id',
-                                    is_gw_port=False)
+            assert_called_once_with(
+                'router-port-id', 'lrp-router-port-id', is_gw_port=False,
+                lsp_address=ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER)
         self.bump_rev_p.assert_called_once_with(self.fake_router_port,
                                                 ovn_const.TYPE_ROUTER_PORTS)
 
@@ -250,8 +256,9 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         self.assertItemsEqual(fake_rtr_intf_networks,
                               called_args_dict.get('networks', []))
         self.l3_inst._ovn.set_lrouter_port_in_lswitch_port.\
-            assert_called_once_with('router-port-id', 'lrp-router-port-id',
-                                    is_gw_port=False)
+            assert_called_once_with(
+                'router-port-id', 'lrp-router-port-id', is_gw_port=False,
+                lsp_address=ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER)
 
     @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_port')
     def test_remove_router_interface(self, getp):
@@ -277,7 +284,9 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             if_exists=False, name='lrp-router-port-id',
             ipv6_ra_configs={},
             networks=['10.0.0.100/24'],
-            external_ids={ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1'})
+            external_ids={
+                ovn_const.OVN_SUBNET_EXT_IDS_KEY: 'subnet-id',
+                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1'})
 
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
                 'update_router')
@@ -400,8 +409,9 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
                           ovn_const.OVN_ROUTER_IS_EXT_GW: 'true',
                           ovn_const.OVN_SUBNET_EXT_ID_KEY: 'ext-subnet-id'})]
         self.l3_inst._ovn.set_lrouter_port_in_lswitch_port.\
-            assert_called_once_with('gw-port-id', 'lrp-gw-port-id',
-                                    is_gw_port=True)
+            assert_called_once_with(
+                'gw-port-id', 'lrp-gw-port-id', is_gw_port=True,
+                lsp_address=ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER)
         self.l3_inst._ovn.add_static_route.assert_has_calls(expected_calls)
 
         bump_rev_calls = [mock.call(self.fake_ext_gw_port,
@@ -445,8 +455,9 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         self.l3_inst._ovn.add_lrouter_port.assert_called_once_with(
             **self.fake_router_port_assert)
         self.l3_inst._ovn.set_lrouter_port_in_lswitch_port.\
-            assert_called_once_with('router-port-id', 'lrp-router-port-id',
-                                    is_gw_port=False)
+            assert_called_once_with(
+                'router-port-id', 'lrp-router-port-id', is_gw_port=False,
+                lsp_address=ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER)
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-router-id', logical_ip='10.0.0.0/24',
             external_ip='192.168.1.1', type='snat')
@@ -475,8 +486,9 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         self.l3_inst._ovn.add_lrouter_port.assert_called_once_with(
             **self.fake_router_port_assert)
         self.l3_inst._ovn.set_lrouter_port_in_lswitch_port.\
-            assert_called_once_with('router-port-id', 'lrp-router-port-id',
-                                    is_gw_port=False)
+            assert_called_once_with(
+                'router-port-id', 'lrp-router-port-id', is_gw_port=False,
+                lsp_address=ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER)
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_not_called()
 
     @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_port')
@@ -522,8 +534,9 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         self.l3_inst._ovn.add_lrouter_port.assert_called_once_with(
             **self.fake_ext_gw_port_assert)
         self.l3_inst._ovn.set_lrouter_port_in_lswitch_port.\
-            assert_called_once_with('gw-port-id', 'lrp-gw-port-id',
-                                    is_gw_port=True)
+            assert_called_once_with(
+                'gw-port-id', 'lrp-gw-port-id', is_gw_port=True,
+                lsp_address=ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER)
         self.l3_inst._ovn.add_static_route.assert_called_once_with(
             'neutron-router-id', ip_prefix='0.0.0.0/0',
             external_ids={ovn_const.OVN_ROUTER_IS_EXT_GW: 'true',
@@ -576,8 +589,9 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         self.l3_inst._ovn.add_lrouter_port.assert_called_once_with(
             **self.fake_ext_gw_port_assert)
         self.l3_inst._ovn.set_lrouter_port_in_lswitch_port.\
-            assert_called_once_with('gw-port-id', 'lrp-gw-port-id',
-                                    is_gw_port=True)
+            assert_called_once_with(
+                'gw-port-id', 'lrp-gw-port-id', is_gw_port=True,
+                lsp_address=ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER)
         self.l3_inst._ovn.add_static_route.assert_called_once_with(
             'neutron-router-id', ip_prefix='0.0.0.0/0',
             nexthop='192.168.1.254',
@@ -625,8 +639,9 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         self.l3_inst._ovn.add_lrouter_port.assert_called_once_with(
             **self.fake_ext_gw_port_assert)
         self.l3_inst._ovn.set_lrouter_port_in_lswitch_port.\
-            assert_called_once_with('gw-port-id', 'lrp-gw-port-id',
-                                    is_gw_port=True)
+            assert_called_once_with(
+                'gw-port-id', 'lrp-gw-port-id', is_gw_port=True,
+                lsp_address=ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER)
         self.l3_inst._ovn.add_static_route.assert_called_once_with(
             'neutron-router-id', ip_prefix='0.0.0.0/0',
             nexthop='192.168.1.254',
@@ -788,13 +803,42 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             ovn_const.OVN_FIP_PORT_EXT_ID_KEY:
                 self.fake_floating_ip['port_id'],
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
-                self.fake_floating_ip['router_id'])}
+                self.fake_floating_ip['router_id']),
+            ovn_const.OVN_FIP_EXT_MAC_KEY: '00:01:02:03:04:05'}
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-router-id', type='dnat_and_snat', logical_ip='10.0.0.10',
             external_ip='192.168.0.10', external_mac='00:01:02:03:04:05',
             logical_port='port_id',
             external_ids=expected_ext_ids)
 
+    @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_port')
+    @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
+    def test_create_floatingip_distributed_logical_port_down(self, gf, gp):
+        # Check that when the port is down, the external_mac field is not
+        # populated. This falls back to centralized routing for ports that
+        # are not bound to a chassis.
+        self.l3_inst._ovn.is_col_present.return_value = True
+        self.l3_inst._ovn.lsp_get_up.return_value.execute.return_value = (
+            False)
+        gp.return_value = {'mac_address': '00:01:02:03:04:05'}
+        gf.return_value = {'floating_port_id': 'fip-port-id'}
+        config.cfg.CONF.set_override(
+            'enable_distributed_floating_ip', True, group='ovn')
+        self.l3_inst.create_floatingip(self.context, 'floatingip')
+        expected_ext_ids = {
+            ovn_const.OVN_FIP_EXT_ID_KEY: self.fake_floating_ip['id'],
+            ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
+            ovn_const.OVN_FIP_PORT_EXT_ID_KEY:
+                self.fake_floating_ip['port_id'],
+            ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
+                self.fake_floating_ip['router_id']),
+            ovn_const.OVN_FIP_EXT_MAC_KEY: '00:01:02:03:04:05'}
+        self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
+            'neutron-router-id', type='dnat_and_snat', logical_ip='10.0.0.10',
+            external_ip='192.168.0.10',
+            logical_port='port_id',
+            external_ids=expected_ext_ids)
+
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
     def test_create_floatingip_external_ip_present_in_nat_rule(self, gf):
         self.l3_inst._ovn.is_col_present.return_value = True
@@ -929,7 +973,8 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             ovn_const.OVN_FIP_PORT_EXT_ID_KEY:
                 self.fake_floating_ip_new['port_id'],
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
-                self.fake_floating_ip_new['router_id'])}
+                self.fake_floating_ip_new['router_id']),
+            ovn_const.OVN_FIP_EXT_MAC_KEY: '00:01:02:03:04:05'}
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-new-router-id', type='dnat_and_snat',
             logical_ip='10.10.10.10', external_ip='192.168.0.10',
@@ -1023,6 +1068,43 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
 
         update_rp_mock.assert_called_once_with(kwargs['port'], if_exists=True)
 
+    @mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin.update_port_status')
+    @mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin.update_port')
+    @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_ports')
+    def test_update_router_gateway_port_bindings_active(
+            self, mock_get_port, mock_updt_port, mock_updt_status):
+        fake_host = 'fake-host'
+        fake_router = 'fake-router'
+        fake_port_id = 'fake-port-id'
+        mock_get_port.return_value = [{
+            'id': fake_port_id,
+            'status': constants.PORT_STATUS_DOWN}]
+        self.l3_inst.update_router_gateway_port_bindings(
+            fake_router, fake_host)
+
+        # Assert that the port is being bound
+        expected_update = {'port': {portbindings.HOST_ID: fake_host}}
+        mock_updt_port.assert_called_once_with(
+            mock.ANY, fake_port_id, expected_update)
+
+        # Assert that the port status is being set to ACTIVE
+        mock_updt_status.assert_called_once_with(
+            mock.ANY, fake_port_id, constants.PORT_STATUS_ACTIVE)
+
+    @mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin.update_port_status')
+    @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_ports')
+    def test_update_router_gateway_port_bindings_down(
+            self, mock_get_port, mock_updt_status):
+        fake_port_id = 'fake-port-id'
+        mock_get_port.return_value = [{
+            'id': fake_port_id,
+            'status': constants.PORT_STATUS_ACTIVE}]
+        self.l3_inst.update_router_gateway_port_bindings(None, None)
+
+        # Assert that the port status is being set to DOWN
+        mock_updt_status.assert_called_once_with(
+            mock.ANY, fake_port_id, constants.PORT_STATUS_DOWN)
+
 
 class OVNL3ExtrarouteTests(test_l3_gw.ExtGwModeIntTestCase,
                            test_l3.L3NatDBIntTestCase,
@@ -1065,6 +1147,10 @@ class OVNL3ExtrarouteTests(test_l3_gw.ExtGwModeIntTestCase,
             'networking_ovn.l3.l3_ovn_scheduler.'
             'OVNGatewayScheduler._schedule_gateway',
             return_value='hv1')
+        self._start_mock(
+            'networking_ovn.common.ovn_client.'
+            'OVNClient.get_candidates_for_scheduling',
+            return_value=[])
         self._start_mock(
             'networking_ovn.common.ovn_client.OVNClient.'
             '_get_v4_network_of_all_router_ports',
